@@ -1,7 +1,32 @@
 ## Geo::Weather
-## Written by Mike Machado <mike@innercite.com> 2002-06-15
+## Written by Mike Machado <mike@innercite.com> 2000-11-01
 ##
 
+# Modified by Kevin L. Papendick
+# E-mail:	kevinp@polarlava.com
+# Website:	www.polarlava.com
+
+# V0.9b
+# - Added report_raw() function
+# - Modified report() format
+
+# V0.9c
+# - URL & RegEx Changes due to weather.com changes
+
+# V1.1_PL
+# - Incorporated Mike's V1.1 $ERROR_BUSY changes
+
+# V1.2
+#  Parse new weather.com as of 2002-12-05
+# - New image locator comment
+# - New current temperature locator
+# - New dew point locator
+# - New relative humidity locator
+# - New visability locator
+# - New barometric locator
+# - New UV locator
+# - New wind locator
+
 package Geo::Weather;
 
 use strict;
@@ -12,11 +37,11 @@ use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK
 		 $OK $ERROR_UNKNOWN $ERROR_QUERY $ERROR_PAGE_INVALID $ERROR_CONNECT $ERROR_NOT_FOUND $ERROR_TIMEOUT $ERROR_BUSY);
 
 require Exporter;
- 
+
 @ISA = qw(Exporter);
 @EXPORT_OK = qw();
 @EXPORT = qw( $OK $ERROR_UNKNOWN $ERROR_QUERY $ERROR_PAGE_INVALID $ERROR_CONNECT $ERROR_NOT_FOUND $ERROR_TIMEOUT $ERROR_BUSY);
-$VERSION = '1.1';
+$VERSION = '1.2';
 
 $OK = 1;
 $ERROR_UNKNOWN = 0;
@@ -27,6 +52,7 @@ $ERROR_NOT_FOUND = -4;
 $ERROR_TIMEOUT = -5;
 $ERROR_BUSY = -6;
 
+
 sub new {
 	my $class = shift;
 	my $self = {};
@@ -34,7 +60,8 @@ sub new {
 	$self->{version} = $VERSION;
 	$self->{server} = 'www.weather.com';
 	$self->{port} = 80;
-	$self->{base} = '/search/search?what=WeatherLocalUndeclared&';
+	$self->{base} = '/search/search?where=';
+	$self->{ext} = '&what=WeatherLocalUndeclared&GO=GO';
 	$self->{timeout} = 10;
 	$self->{proxy} = '';
 	$self->{proxy_username} = '';
@@ -55,13 +82,13 @@ sub get_weather {
 	my $page = '';
 	if ($city =~ /^\d+$/) {
 		# Use zip code
-		$page = $self->{base}.'where='.$city;
+		$page = $self->{base}.$city.$self->{ext};
 	} else {
 		# Use state_city
 		$state = lc($state);
 		$city = lc($city);
 		$city =~ s/ /+/g;
-		$page = $self->{base}.'where='.$city.','.$state;
+		$page = $self->{base}.$city.','.$state.$self->{ext};
 	}
 
 	$self->{results} = $self->lookup($page);
@@ -69,35 +96,171 @@ sub get_weather {
 	return $self->{results};
 }
 
+sub report_raw {
+	my $self = shift;
+	my $results = $self->{results};
+	my $output;
+
+	return $ERROR_UNKNOWN unless $self->{results};
+
+	$output .= $results->{city} . '|';
+	$output .= $results->{state} . '|';
+	$output .= $results->{pic} . '|';
+	$output .= $results->{cond} . '|';
+	$output .= $results->{temp} . '|';
+	$output .= $results->{wind} . '|';
+	$output .= $results->{dewp} . '|';
+	$output .= $results->{humi} . '|';
+	$output .= $results->{visb} . '|';
+	$output .= $results->{baro} . '|';
+	$output .= $results->{uv};
+
+	return "$output";
+}
+
 sub report {
 	my $self = shift;
+	my $result_hdr_color = "#0080FF";
+	my $result_cond_color = "#FF8080";
+	my $result_color = "#03C1C7";
 
 	return $ERROR_UNKNOWN unless $self->{results};
 
 	my $output = '';
+	my $heat_c = 0;
+	my $feels_like = '';
 	my $results = $self->{results};
-	$output .= "<font size=+4>$results->{city}, $results->{state}</font><br>\n";
-	$output .= "<a href=\"$results->{url}\"><img src=\"$results->{pic}\" border=0></a>\n";
-	$output .= "<font size=+3>$results->{cond}</font><br>\n";
-	$output .= "<table border=0>\n";
-	$output .= "<tr><td><b>Temp</b></td><td>$results->{temp}&deg F</td>\n";
-	$output .= "<tr><td><b>Heat Index</b></td><td>$results->{heat}&deg F</td>\n";
-	$output .= "<tr><td><b>Wind</b></td><td>$results->{wind}</td>\n" if $results->{wind};
-	$output .= "<tr><td><b>Dew Point</b></td><td>$results->{dewp}&deg F</td>\n";
-	$output .= "<tr><td><b>Rel. Humidity</b></td><td>$results->{humi}</td>\n";
-	$output .= "<tr><td><b>Visibility</b></td><td>$results->{visb}</td>\n";
-	$output .= "<tr><td><b>Barometer</b></td><td>$results->{baro}</td>\n" if $results->{baro};
+
+	if ($results->{heat} ne 'N/A') {
+		$heat_c = sprintf("%0.0f", 5/9 * ($results->{heat} - 32));
+		$feels_like = "(Feels Like: $results->{heat}&deg F/$heat_c&deg C)";
+	}
+
+
+	$output = <<REPORT_START;
+	<font size="+2" color=\"$result_hdr_color\">
+		$results->{city}, $results->{state}
+	</font>
+	<br>
+	<a href=\"$results->{url}\"><img src=\"$results->{pic}\" border=0></a>
+	<font size="+1" color="$result_cond_color">
+		$results->{cond}
+	</font>
+	<br>
+	<br>
+	<table border="0">
+		<tr>
+			<td>
+					<b>Temperature:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{temp}&deg F/$results->{temp_c}&deg C&nbsp;&nbsp; $feels_like
+				</font>
+			</td>
+		</tr>
+
+REPORT_START
+
+	if ($results->{wind}) {
+		$output .= <<REPORT_WIND;
+		<tr>
+			<td>
+				<b>Wind:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{wind}
+				</font>
+			</td>
+		</tr>
+
+REPORT_WIND
+	}
+
+	$output .= <<REPORT_MID;
+		<tr>
+			<td>
+				<b>Dew Point:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{dewp}&deg F/$results->{dewp_c}&deg C
+				</font>
+			</td>
+		</tr>
+		<tr>
+			<td>
+				<b>Rel. Humidity:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{humi} %
+				</font>
+			</td>
+		</tr>
+		<tr>
+			<td>
+				<b>Visibility:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{visb}
+				</font>
+			</td>
+		</tr>
+
+REPORT_MID
+
+	if ($results->{baro}) {
+		$output .= <<REPORT_BARO;
+		<tr>
+			<td>
+				<b>Barometer:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{baro}
+				</font>
+			</td>
+		</tr>
+
+REPORT_BARO
+	}
+
+	if ($results->{baro}) {
+		$output .= <<REPORT_UV;
+		<tr>
+			<td>
+				<b>UV Index:</b>
+			</td>
+			<td>
+				<font color="$result_color">
+					$results->{uv}
+				</font>
+			</td>
+		</tr>
+
+REPORT_UV
+	}
+
 	$output .= "</table>\n";
 
-	return $output;
+	return "$output|$results->{city}|$results->{state}";
 }
-	
 
 sub lookup {
 	my $self = shift;
 	my $page = shift || '';
 	my $redir = shift || 0;
 
+	my $rh_cnt = 0;
+	my $dew_cnt = 0;
+	my $vis_cnt = 0;
+	my $baro_cnt = 0;
+	my $uv_cnt = 0;
+	my $wind_cnt = 0;
+
 	return $ERROR_PAGE_INVALID unless $page;
 
 	my %results = ();
@@ -107,7 +270,8 @@ sub lookup {
 	$results{url} .= $page;
 	$results{page} = $page;
 
-	my $not_found_marker = 'could not be found';
+	my $not_found_marker = 'not found';
+	my $end_report_marker = '<!-- insert reported/last updated info -->';
 	my $line = '';
 
 	print STDERR __LINE__, ": Geo::Weather: Attempting to GET $results{url}\n" if $self->{debug};
@@ -120,36 +284,27 @@ sub lookup {
 	$ua->timeout($self->{timeout}) if $self->{timeout};
 	$ua->agent($self->{agent_string});
 	$ua->proxy(['http'], $self->{proxy}) if $self->{proxy};
-	
+
 
 	my $response = $ua->request($request);
 	unless ($response->is_success) {
 		return $ERROR_TIMEOUT;
 	}
 	my $content = $response->content();
-
 	my @lines = split(/\n/, $content);
 	for (my $i = 0; $i < @lines; $i++) {
 		my $line = $lines[$i];
-
-		print STDERR "tagline: $i: $line\n" if ($line =~ /<!-- insert/ && $self->{debug} > 2);
+		next if ($line eq '');
+		print STDERR "tagline: $line\n" if ($line =~ /<!-- insert/ && $self->{debug} > 2);
 		print STDERR "line: $line\n" if $self->{debug} > 3;
 
 		return $ERROR_NOT_FOUND if ($line =~ /$not_found_marker/i);
 
-		if ($line =~ /categoryTitle/) {
-			$line = $lines[$i + 2];
-			if ($line =~ /\"(.*?)\"/) {
-				print STDERR __LINE__, ": Geo::Weather: Found search result $1\n" if $self->{debug};
-				return $self->lookup($1);
-			}
-		}
-
 		if ($line =~ /<title>.*Severe Weather Mode Index.*/i) {
 			return $ERROR_BUSY;
 		}
 
-		if ($line =~ /<b>Local Forecast for (.*?)<\/b>/i) {
+		if ($line =~ /<b>Local Forecast for (.*?)<\/b>/i || $line =~ /<b>Travel Forecast for (.*?)<\/b>/i) {
 			my ($city, $state) = split(/\,[\s+]/, $1);
 			$results{city} = $city;
 			if ($state =~ /(.*)\s+\((.*)\)/) {
@@ -159,62 +314,119 @@ sub lookup {
 				$results{state} = $state;
 			}
 		}
+
 		if (!$results{pic}) {
-			if ($line =~ /<!-- insert current weather icon -->\s*<img src=\"(.*?)\"/i) {
+			if ($line =~ /<!-- http:\/\/image.weather.com\/web\/common\/wxicons\/52\/666.gif -->\s*<img src=\"(.*?)\"/i) {
 				$results{pic} = $1;
 			}
 		}
 		if (!$results{cond}) {
-			if ($line =~ /<!-- insert forecast -->(.*?)[<&]/) {
+			if ($line =~ /<!-- insert forecast -->(.*?)\s*[<&]/) {
 				$results{cond} = $1;
 			}
 		}
 		if (!$results{temp}) {
-			if ($line =~ /<!-- insert current tempature --.*>(.*?)(<|&deg)/) {
+			if ($line =~ /obsTempTextBlue\"><\w+>\s*(.*?)[<&]/i) {
 				$results{temp} = $1;
 			}
 		}
 		if (!$results{heat}) {
-			if ($line =~ /<!-- insert feels like temp -->(.*?)[<&]/) {
+			if ($line =~ /Feels Like<\w+>\s*(.*?)[<&]/) {
 				$results{heat} = $1;
 			}
 		}
+
 		if (!$results{uv}) {
-			if ($line =~ /<!-- insert UV index -->(.*?)[<&]/) {
+			if ($line =~ /UV Index:/) {
+				$uv_cnt = 1;
+			} elsif ($uv_cnt > 0) {
+				$uv_cnt++;
+			}
+
+			if ($uv_cnt == 3 && $line =~ /obsTextBlue\">\s*(.*)</) {
+
 				$results{uv} = $1;
 			}
 		}
+
 		if (!$results{wind}) {
-			if ($line =~ /<!-- insert wind -->\s*(.*?)[<&]/) {
+			if ($line =~ /Wind:/) {
+				$wind_cnt = 1;
+			} elsif ($wind_cnt > 0) {
+				$wind_cnt++;
+			}
+
+			if ($wind_cnt == 3 && $line =~ /obsTextBlue\">\s*(.*)</) {
+
 				$results{wind} = $1;
 			}
 		}
+
 		if (!$results{dewp}) {
-			if ($line =~ /<!-- insert dew point -->(.*?)[<&]/) {
+			if ($line =~ /Dew Point:/) {
+				$dew_cnt = 1;
+			} elsif ($dew_cnt > 0) {
+				$dew_cnt++;
+			}
+
+			if ($dew_cnt == 3 && $line =~ /obsTextBlue\">\s*(\d+)/) {
+
 				$results{dewp} = $1;
 			}
 		}
+
 		if (!$results{humi}) {
-			if ($line =~ /<!-- insert humidity -->(.*?)[\s<&]/) {
+			if ($line =~ /Humidity:/) {
+				$rh_cnt = 1;
+			} elsif ($rh_cnt > 0) {
+				$rh_cnt++;
+			}
+
+			if ($rh_cnt == 3 && $line =~ /obsTextBlue\">\s*(\d+)/) {
+
 				$results{humi} = $1;
 			}
 		}
+
 		if (!$results{visb}) {
-			if ($line =~ /<!-- insert visibility -->(.*?)[<&]/) {
+			if ($line =~ /Visibility:/) {
+				$vis_cnt = 1;
+			} elsif ($vis_cnt > 0) {
+				$vis_cnt++;
+			}
+
+			if ($vis_cnt == 3 && $line =~ /obsTextBlue\">\s*(.*)\s*</) {
+
 				$results{visb} = $1;
 			}
 		}
+
 		if (!$results{baro}) {
-			if ($line =~ /<!-- insert pressure -->(.*?)[<&]/) {
+			if ($line =~ /Pressure:/) {
+				$baro_cnt = 1;
+			} elsif ($baro_cnt > 0) {
+				$baro_cnt++;
+			}
+
+			if ($baro_cnt == 3 && $line =~ /obsTextBlue\">\s*(.*)\s*</) {
+
 				$results{baro} = $1;
 			}
 		}
 
+
+		if ($line =~ /$end_report_marker/) {
+			last;
+		}
 	}
 	if (!($results{visb})) {
 		$results{visb} = 'Not Available';
 	}
 
+	$results{temp_c} = sprintf("%0.0f", 5/9 * ($results{temp} - 32));
+	$results{dewp_c} = sprintf("%0.0f", 5/9 * ($results{dewp} - 32));
+
+
 	return \%results;
 }
 
@@ -242,7 +454,7 @@ Geo::Weather - Weather retrieval module
 
   my $weather = new Geo::Weather;
   $weather->{timeout} = 5; # set timeout to 5 seconds instead of the default of 10
- 
+
   my $current = $weather->get_weather('95630');
 
   print "The current temperature is $current->{temp} degrees\n";
@@ -300,16 +512,15 @@ B<Returns>
 	baro		- Current barometric pressure
 	heat		- Current heat index (Feels Like string)
 
-B<Errors>
+	On error, it returns the following exported error variables
 
-	On error, it returns one of the following exported error variables
+B<Errors>
 
 	$ERROR_QUERY		- Invalid data supplied
 	$ERROR_PAGE_INVALID	- No URL, or incorrectly formatted URL for retrieving the information
 	$ERROR_CONNECT		- Error connecting to weather.com
 	$ERROR_NOT_FOUND	- Weather for the specified city/state or zip could not be found
 	$ERROR_TIMEOUT		- Timed out while trying to connect or get data from weather.com
-	$ERROR_BUSY		- weather.com is too busy to handle requests and is in Severe Weather Alert Mode
 
 =back
 
@@ -321,12 +532,26 @@ Returns an HTML table containing the current weather. Must call get_weather firs
 
 B<Sample Code>
 
-	$weather->get_weather('90210');
 	print $weather->report();
 
 =back
 
 
+=over 4
+
+=item * B<report_raw>
+
+Returns pipe delimited string containing the current weather. Must call get_weather first.
+
+ Fields are: city|state|pic|cond|temp|wind|dewp|humi|visb|baro|uv
+
+B<Sample Code>
+
+	my $current = $weather->report_raw();
+
+=back
+
+
 =over 4
 
 =item * B<lookup>
@@ -373,7 +598,7 @@ Sets the username to use for proxying. Defaults to the HTTP_PROXY_USER environme
 
 Sets the password to use for proxying. Defaults to the HTTP_PROXY_PASS environment variable, if set.
 
-=item * B<agent_string>
+=item *B<agent_string>
 
 HTTP User-Agent header for request. Default is Geo::Weather/$VERSION.
 
